#include <fcntl.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <vector>

#include "client.h"
#include "serverpool.h"
#include "stats.h"

using namespace std;

extern ServerPool *servers;

StatsThread::StatsThread(const std::string &stats_file, int stats_interval)
	: stats_file(stats_file),
	  stats_interval(stats_interval)
{
}

void StatsThread::do_work()
{
	while (!should_stop) {
		int fd;
		FILE *fp;
		time_t now;
		vector<ClientStats> client_stats;

		// Open a new, temporary file.
		char *filename = strdup((stats_file + ".new.XXXXXX").c_str());
		fd = mkostemp(filename, O_WRONLY);
		if (fd == -1) {
			perror(filename);
			free(filename);
			goto sleep;
		}

		fp = fdopen(fd, "w");
		if (fp == NULL) {
			perror("fdopen");
			close(fd);
			unlink(filename);
			free(filename);
			goto sleep;
		}

		now = time(NULL);
		client_stats = servers->get_client_stats();
		for (size_t i = 0; i < client_stats.size(); ++i) {
			fprintf(fp, "%s %s %d %llu\n",
			        client_stats[i].remote_addr.c_str(),
				client_stats[i].stream_id.c_str(),
				int(now - client_stats[i].connect_time),
				(long long unsigned)(client_stats[i].bytes_sent));
		}
		if (fclose(fp) == EOF) {
			perror("fclose");
			unlink(filename);
			free(filename);
			goto sleep;
		}
		
		if (rename(filename, stats_file.c_str()) == -1) {
			perror("rename");
			unlink(filename);
		}

sleep:
		int left_to_sleep = stats_interval;
		do {
			left_to_sleep = sleep(left_to_sleep);
		} while (left_to_sleep > 0 && !should_stop);
	}
}
